@@ -19,12 +19,17 @@ PGrnInitializeBuffers(void)
 	GRN_UINT64_INIT(&(PGrnBuffers.ctid), 0);
 	GRN_FLOAT_INIT(&(PGrnBuffers.score), 0);
 	GRN_UINT32_INIT(&(PGrnBuffers.sourceIDs), GRN_OBJ_VECTOR);
+	GRN_TEXT_INIT(&(PGrnBuffers.tokenizer), GRN_OBJ_DO_SHALLOW_COPY);
+	GRN_TEXT_INIT(&(PGrnBuffers.normalizer), GRN_OBJ_DO_SHALLOW_COPY);
 	GRN_PTR_INIT(&(PGrnBuffers.tokenFilters), GRN_OBJ_VECTOR, GRN_ID_NIL);
+	GRN_TEXT_INIT(&(PGrnBuffers.text), 0);
+	GRN_TEXT_INIT(&(PGrnBuffers.texts), GRN_OBJ_VECTOR);
 	GRN_UINT64_INIT(&(PGrnBuffers.jsonbValueKeys), GRN_OBJ_VECTOR);
 	GRN_UINT64_INIT(&(PGrnBuffers.walPosition), 0);
 	GRN_VOID_INIT(&(PGrnBuffers.walValue));
 	GRN_UINT32_INIT(&(PGrnBuffers.maxRecordSize), 0);
 	GRN_UINT64_INIT(&(PGrnBuffers.walAppliedPosition), 0);
+	GRN_BOOL_INIT(&(PGrnBuffers.isTargets), GRN_OBJ_VECTOR);
 	GRN_TEXT_INIT(&(PGrnBuffers.escape.escapedValue), 0);
 	GRN_TEXT_INIT(&(PGrnBuffers.escape.specialCharacters), 0);
 	GRN_TEXT_INIT(&(PGrnBuffers.head), 0);
@@ -43,12 +48,17 @@ PGrnFinalizeBuffers(void)
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.ctid));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.score));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.sourceIDs));
+	GRN_OBJ_FIN(ctx, &(PGrnBuffers.tokenizer));
+	GRN_OBJ_FIN(ctx, &(PGrnBuffers.normalizer));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.tokenFilters));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.jsonbValueKeys));
+	GRN_OBJ_FIN(ctx, &(PGrnBuffers.text));
+	GRN_OBJ_FIN(ctx, &(PGrnBuffers.texts));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.walPosition));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.walValue));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.maxRecordSize));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.walAppliedPosition));
+	GRN_OBJ_FIN(ctx, &(PGrnBuffers.isTargets));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.escape.escapedValue));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.escape.specialCharacters));
 	GRN_OBJ_FIN(ctx, &(PGrnBuffers.head));
