@@ -11,6 +11,10 @@
 #endif
 
 #ifdef PGRN_SUPPORT_OPTIONS
+static const char *PGRN_LEXICON_TYPE_HASH_TABLE = "hash_table";
+static const char *PGRN_LEXICON_TYPE_PATRICIA_TRIE = "patricia_trie";
+static const char *PGRN_LEXICON_TYPE_DOUBLE_ARRAY_TRIE = "double_array_trie";
+
 typedef struct PGrnOptions
 {
 	int32 vl_len_;
@@ -21,11 +25,14 @@ typedef struct PGrnOptions
 	int fullTextSearchNormalizerOffset;
 	int regexpSearchNormalizerOffset;
 	int prefixSearchNormalizerOffset;
+	int lexiconTypeOffset;
 } PGrnOptions;
 
 static relopt_kind PGrnReloptionKind;
 
 static grn_ctx *ctx = &PGrnContext;
+static struct PGrnBuffers *buffers = &PGrnBuffers;
+static grn_obj *lexicon = NULL;
 
 PGRN_FUNCTION_INFO_V1(pgroonga_options);
 
@@ -65,22 +72,27 @@ PGrnOptionParseNames(const char *names,
 	}
 }
 
-static bool
-PGrnIsTokenizer(grn_obj *object)
+static void
+PGrnOptionEnsureLexicon(const char *context)
 {
-	if (object->header.type != GRN_PROC)
-		return false;
+	if (lexicon)
+		grn_obj_close(ctx, lexicon);
 
-	if (grn_proc_get_type(ctx, object) != GRN_PROC_TOKENIZER)
-		return false;
-
-	return true;
+	lexicon = grn_table_create(ctx,
+							   NULL, 0,
+							   NULL,
+							   GRN_TABLE_PAT_KEY,
+							   grn_ctx_at(ctx, GRN_DB_SHORT_TEXT),
+							   NULL);
+	PGrnCheck("options: %s: failed to create a temporary lexicon",
+			  context);
 }
 
 static void
 PGrnOptionValidateTokenizer(char *name)
 {
-	grn_obj *tokenizer;
+	grn_obj *tokenizer_name = &(buffers->tokenizer);
+	grn_rc rc;
 
 	if (PGrnIsNoneValue(name))
 		return;
@@ -88,40 +100,27 @@ PGrnOptionValidateTokenizer(char *name)
 	if (strcmp(name, PGRN_DEFAULT_TOKENIZER) == 0)
 		return;
 
-	tokenizer = grn_ctx_get(ctx, name, -1);
-	if (!tokenizer)
-	{
+	PGrnOptionEnsureLexicon("tokenizer");
+
+	GRN_TEXT_SETS(ctx, tokenizer_name, name);
+	rc = grn_obj_set_info(ctx,
+						  lexicon,
+						  GRN_INFO_DEFAULT_TOKENIZER,
+						  tokenizer_name);
+	if (rc != GRN_SUCCESS) {
 		ereport(ERROR,
 				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-				 errmsg("pgroonga: nonexistent tokenizer: <%s>",
-						name)));
+				 errmsg("pgroonga: invalid tokenizer: <%s>: %s",
+						name,
+						ctx->errbuf)));
 	}
-
-	if (!PGrnIsTokenizer(tokenizer))
-	{
-		ereport(ERROR,
-				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-				 errmsg("pgroonga: not tokenizer: <%s>: %s",
-						name, PGrnInspect(tokenizer))));
-	}
-}
-
-static bool
-PGrnIsNormalizer(grn_obj *object)
-{
-	if (object->header.type != GRN_PROC)
-		return false;
-
-  if (grn_proc_get_type(ctx, object) != GRN_PROC_NORMALIZER)
-	  return false;
-
-  return true;
 }
 
 static void
 PGrnOptionValidateNormalizer(char *name)
 {
-	grn_obj *normalizer;
+	grn_obj *normalizer_name = &(buffers->normalizer);
+	grn_rc rc;
 
 	if (PGrnIsNoneValue(name))
 		return;
@@ -129,21 +128,19 @@ PGrnOptionValidateNormalizer(char *name)
 	if (strcmp(name, PGRN_DEFAULT_NORMALIZER) == 0)
 		return;
 
-	normalizer = grn_ctx_get(ctx, name, -1);
-	if (!normalizer)
-	{
-		ereport(ERROR,
-				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-				 errmsg("pgroonga: nonexistent normalizer: <%s>",
-						name)));
-	}
+	PGrnOptionEnsureLexicon("normalizer");
 
-	if (!PGrnIsNormalizer(normalizer))
-	{
+	GRN_TEXT_SETS(ctx, normalizer_name, name);
+	rc = grn_obj_set_info(ctx,
+						  lexicon,
+						  GRN_INFO_NORMALIZER,
+						  normalizer_name);
+	if (rc != GRN_SUCCESS) {
 		ereport(ERROR,
 				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-				 errmsg("pgroonga: not normalizer: <%s>: %s",
-						name, PGrnInspect(normalizer))));
+				 errmsg("pgroonga: invalid normalizer: <%s>: %s",
+						name,
+						ctx->errbuf)));
 	}
 }
 
@@ -218,11 +215,38 @@ PGrnOptionValidatePlugins(char *names)
 						 PGrnOptionValidatePlugin,
 						 NULL);
 }
+
+static void
+PGrnOptionValidateLexiconType(char *name)
+{
+	if (!name)
+		return;
+
+	if (strcmp(name, PGRN_LEXICON_TYPE_HASH_TABLE) == 0)
+		return;
+
+	if (strcmp(name, PGRN_LEXICON_TYPE_PATRICIA_TRIE) == 0)
+		return;
+
+	if (strcmp(name, PGRN_LEXICON_TYPE_DOUBLE_ARRAY_TRIE) == 0)
+		return;
+
+	ereport(ERROR,
+			(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+			 errmsg("pgroonga: invalid lexicon type: <%s>: "
+					"available types: "
+					"[%s, %s, %s]",
+					name,
+					PGRN_LEXICON_TYPE_HASH_TABLE,
+					PGRN_LEXICON_TYPE_PATRICIA_TRIE,
+					PGRN_LEXICON_TYPE_DOUBLE_ARRAY_TRIE)));
+}
 #endif
 
 void
 PGrnInitializeOptions(void)
 {
+	lexicon = NULL;
 #ifdef PGRN_SUPPORT_OPTIONS
 	PGrnReloptionKind = add_reloption_kind();
 
@@ -262,9 +286,23 @@ PGrnInitializeOptions(void)
 						 "Normalizer name to be used for prefix search",
 						 NULL,
 						 PGrnOptionValidateNormalizer);
+	add_string_reloption(PGrnReloptionKind,
+						 "lexicon_type",
+						 "Lexicon type to be used for lexicon",
+						 NULL,
+						 PGrnOptionValidateLexiconType);
 #endif
 }
 
+void
+PGrnFinalizeOptions(void)
+{
+	if (lexicon)
+	{
+		grn_obj_close(ctx, lexicon);
+	}
+}
+
 #ifdef PGRN_SUPPORT_OPTIONS
 static void
 PGrnOptionCollectTokenFilter(const char *name,
@@ -286,13 +324,15 @@ PGrnApplyOptionValues(Relation index,
 					  const char *defaultTokenizerName,
 					  grn_obj **normalizer,
 					  const char *defaultNormalizerName,
-					  grn_obj *tokenFilters)
+					  grn_obj *tokenFilters,
+					  grn_table_flags *lexiconType)
 {
 #ifdef PGRN_SUPPORT_OPTIONS
 	PGrnOptions *options;
 	const char *tokenizerName;
 	const char *normalizerName;
 	const char *tokenFilterNames;
+	const char *lexiconTypeName;
 
 	options = (PGrnOptions *) (index->rd_options);
 	if (!options)
@@ -307,6 +347,8 @@ PGrnApplyOptionValues(Relation index,
 		else
 			*normalizer = NULL;
 
+		*lexiconType |= GRN_OBJ_TABLE_PAT_KEY;
+
 		return;
 	}
 
@@ -332,7 +374,8 @@ PGrnApplyOptionValues(Relation index,
 		}
 		else
 		{
-			*tokenizer = PGrnLookup(tokenizerName, ERROR);
+			*tokenizer = &(buffers->tokenizer);
+			GRN_TEXT_SETS(ctx, *tokenizer, tokenizerName);
 		}
 	}
 
@@ -377,12 +420,34 @@ PGrnApplyOptionValues(Relation index,
 	}
 	else
 	{
-		*normalizer = PGrnLookup(normalizerName, ERROR);
+		*normalizer = &(buffers->normalizer);
+		GRN_TEXT_SETS(ctx, *normalizer, normalizerName);
 	}
 
 	PGrnOptionParseNames(tokenFilterNames,
 						 PGrnOptionCollectTokenFilter,
 						 tokenFilters);
+
+	lexiconTypeName = GET_STRING_RELOPTION(options, lexiconTypeOffset);
+	if (lexiconTypeName)
+	{
+		if (strcmp(lexiconTypeName, PGRN_LEXICON_TYPE_HASH_TABLE) == 0)
+		{
+			*lexiconType |= GRN_OBJ_TABLE_HASH_KEY;
+		}
+		else if (strcmp(lexiconTypeName, PGRN_LEXICON_TYPE_PATRICIA_TRIE) == 0)
+		{
+			*lexiconType |= GRN_OBJ_TABLE_PAT_KEY;
+		}
+		else if (strcmp(lexiconTypeName, PGRN_LEXICON_TYPE_DOUBLE_ARRAY_TRIE) == 0)
+		{
+			*lexiconType |= GRN_OBJ_TABLE_DAT_KEY;
+		}
+	}
+	else
+	{
+		*lexiconType |= GRN_OBJ_TABLE_PAT_KEY;
+	}
 #endif
 }
 
@@ -408,14 +473,21 @@ pgroonga_options_raw(Datum reloptions,
 		{"regexp_search_normalizer", RELOPT_TYPE_STRING,
 		 offsetof(PGrnOptions, regexpSearchNormalizerOffset)},
 		{"prefix_search_normalizer", RELOPT_TYPE_STRING,
-		 offsetof(PGrnOptions, prefixSearchNormalizerOffset)}
+		 offsetof(PGrnOptions, prefixSearchNormalizerOffset)},
+		{"lexicon_type", RELOPT_TYPE_STRING,
+		 offsetof(PGrnOptions, lexiconTypeOffset)}
 	};
 
 	options = parseRelOptions(reloptions, validate, PGrnReloptionKind,
 							  &nOptions);
 	grnOptions = allocateReloptStruct(sizeof(PGrnOptions), options, nOptions);
-	fillRelOptions(grnOptions, sizeof(PGrnOptions), options, nOptions,
-				   validate, optionsMap, lengthof(optionsMap));
+	fillRelOptions(grnOptions,
+				   sizeof(PGrnOptions),
+				   options,
+				   nOptions,
+				   validate,
+				   optionsMap,
+				   lengthof(optionsMap));
 	pfree(options);
 
 	return (bytea *) grnOptions;
